@@ -203,6 +203,7 @@ void create_tables()
     exec_query(
             "insert into wiki(id, name, content, registerdate) values (NULL, 'help', '"
             "*機能説明\n"
+            "各ページの説明です。\n"
             "**トップ\n"
             "ナビゲーションメニューから移動できます。\n"
             "***簡易ページ編集機能\n"
@@ -294,13 +295,16 @@ void create_tables()
             "**RSS\n"
             "最新情報をRSS feedを配信します。\n"
             "最近更新のあった10件についての情報を配信します。\n"
+            "**チケット一括登録\n"
+            "複数のチケットを一括登録します。\n"
+            "タスク管理に利用する場合など、登録するチケットが多数あり予めわかっている場合に一括でチケットを登録できます。\n"
             "**ヘルプ\n"
             "ナビゲーションメニューから移動できます。\n"
             "ヘルプを表示します。\n"
             "**管理ツール\n"
             "管理ツールメニューへ移動します。\n"
             "\n"
-            "*逆引き\n"
+            "*逆引きヘルプ\n"
             "**最新のチケットの情報をCSV形式でダウンロードしたい\n"
             "チケット検索で検索した結果を「検索結果をCSVでダウンロードする。」のリンクからダウンロードできます。\n"
             "**ある状態で進捗のないチケットを調べたい\n"
