@@ -9,6 +9,7 @@
 #include "util.h"
 #include "wiki.h"
 #include "hook.h"
+#include "csv.h"
 #include "simple_string.h"
 
 typedef enum _NAVI {
@@ -16,6 +17,7 @@ typedef enum _NAVI {
     NAVI_TOP,
     NAVI_LIST,
     NAVI_REGISTER,
+    NAVI_REGISTER_AT_ONCE,
     NAVI_SEARCH,
     NAVI_RSS,
     NAVI_HELP,
@@ -32,6 +34,9 @@ typedef enum _MODE {
 void register_actions();
 void list_action();
 void search_actoin();
+void register_at_once_action();
+void register_at_once_confirm_action();
+void register_at_once_submit_action();
 void register_submit_action();
 void register_action();
 void ticket_action();
@@ -67,6 +72,9 @@ void register_actions()
     register_action_actions("list", list_action);
     register_action_actions("search", search_actoin);
     register_action_actions("register", register_action);
+    register_action_actions("register_at_once", register_at_once_action);
+    register_action_actions("register_at_once_confirm", register_at_once_confirm_action);
+    register_action_actions("register_at_once_submit", register_at_once_submit_action);
     register_action_actions("register_submit", register_submit_action);
     register_action_actions("ticket", ticket_action);
     register_action_actions("statistics", statistics_action);
@@ -82,6 +90,9 @@ void register_actions()
 
 void output_header(Project* project, char* title, char* script_name, NaviType navi)
 {
+
+    String* base_url_a = string_new(0);
+    base_url_a = get_base_url(base_url_a);
     o("Pragma: no-cache\r\n");
     o("Cache-Control: no-cache\t\n");
     cgiHeaderContentType("text/html; charset=utf-8;");
@@ -99,21 +110,24 @@ void output_header(Project* project, char* title, char* script_name, NaviType na
         o(  "\t<script type=\"text/javascript\" src=\"%s/../js/prototype.js\"></script>\n", cgiScriptName);
         o(  "\t<script type=\"text/javascript\" src=\"%s/../js/%s\"></script>\n", cgiScriptName, script_name);
     }
+    o(      "\t<link rel=\"alternate\" type=\"application/rss+xml\" title=\"RSS\" href=\"%s/rss\">\n", string_rawstr(base_url_a));
+    string_free(base_url_a);
     o(      "</head>\n"
             "<body>\n"
             "<a name=\"top\"></a>\n"
             "<h1 id=\"toptitle\" title=\"Starbug1\"><a href=\"http://starbug1.sourceforge.jp/\"><img src=\"%s/../img/title.jpg\" alt=\"Starbug1\" /></a></h1>\n", cgiScriptName);
     o("<div>\n"
       "<ul id='menu'>\n");
-    o("<li><a href='%s' title=\"ホームへ移動します\">ホーム</a></li>\n", project->home_url);
-    o("<li><a %s href='%s' title=\"トップページへ移動します\">トップ</a></li>\n", navi == NAVI_TOP ? "class=\"current\"" : "", cgiScriptName);
-    o("<li><a %s href='%s/list' title=\"状態別のチケット一覧を参照します\">状態別チケット一覧</a></li>\n", navi == NAVI_LIST ? "class=\"current\"" : "", cgiScriptName);
-    o("<li><a %s href='%s/register' title=\"新規にチケットを登録します\">チケット登録</a></li>\n", navi == NAVI_REGISTER ? "class=\"current\"" : "", cgiScriptName);
-    o("<li><a %s href='%s/search' title=\"チケットを検索します\">チケット検索</a></li>\n", navi == NAVI_SEARCH ? "class=\"current\"" : "", cgiScriptName);
-    o("<li><a %s href='%s/statistics' title=\"統計情報を表示します\">統計情報</a></li>\n", navi == NAVI_STATISTICS ? "class=\"current\"" : "", cgiScriptName);
-    o("<li><a %s href='%s/rss' title=\"RSS Feed\">RSS Feed</a></li>\n", navi == NAVI_RSS ? "class=\"current\"" : "", cgiScriptName);
-    o("<li><a %s href='%s/help' title=\"ヘルプを参照します\">ヘルプ</a></li>\n", navi == NAVI_HELP ? "class=\"current\"" : "", cgiScriptName);
-    o("<li><a %s href='%s/../admin.cgi' title=\"各種設定を行ないます\">管理ツール</a></li>\n", navi == NAVI_MANAGEMENT ? "class=\"current\"" : "", cgiScriptName);
+    o("\t<li><a href='%s' title=\"ホームへ移動します\">ホーム</a></li>\n", project->home_url);
+    o("\t<li><a %s href='%s' title=\"トップページへ移動します\">トップ</a></li>\n", navi == NAVI_TOP ? "class=\"current\"" : "", cgiScriptName);
+    o("\t<li><a %s href='%s/list' title=\"状態別のチケット一覧を参照します\">状態別チケット一覧</a></li>\n", navi == NAVI_LIST ? "class=\"current\"" : "", cgiScriptName);
+    o("\t<li><a %s href='%s/register' title=\"新規にチケットを登録します\">チケット登録</a></li>\n", navi == NAVI_REGISTER ? "class=\"current\"" : "", cgiScriptName);
+    o("\t<li><a %s href='%s/search' title=\"チケットを検索します\">チケット検索</a></li>\n", navi == NAVI_SEARCH ? "class=\"current\"" : "", cgiScriptName);
+    o("\t<li><a %s href='%s/statistics' title=\"統計情報を表示します\">統計情報</a></li>\n", navi == NAVI_STATISTICS ? "class=\"current\"" : "", cgiScriptName);
+    o("\t<li><a %s href='%s/register_at_once' title=\"複数新規にチケットを登録します\">チケット一括登録</a></li>\n", navi == NAVI_REGISTER_AT_ONCE ? "class=\"current\"" : "", cgiScriptName);
+    o("\t<li><a %s href='%s/rss' title=\"RSS Feed\">RSS Feed</a></li>\n", navi == NAVI_RSS ? "class=\"current\"" : "", cgiScriptName);
+    o("\t<li><a %s href='%s/help' title=\"ヘルプを参照します\">ヘルプ</a></li>\n", navi == NAVI_HELP ? "class=\"current\"" : "", cgiScriptName);
+    o("\t<li><a %s href='%s/../admin.cgi' title=\"各種設定を行ないます\">管理ツール</a></li>\n", navi == NAVI_MANAGEMENT ? "class=\"current\"" : "", cgiScriptName);
     o("</ul>\n"
       "<br clear='all' />\n"
       "</div>\n");
@@ -221,9 +235,9 @@ void output_ticket_table_header_no_link(List* element_types)
     o(      "\t\t<th class=\"id\">ID</th>\n");
     foreach (it, element_types) {
         ElementType* et = it->element;
-        o("\t\t<th class=\"field%d\">\n", et->id);
+        o("\t\t<th class=\"field%d\">", et->id);
         h(et->name);
-        o("\t\t</th>\n");
+        o(    "</th>\n");
     }
     o("\t\t<th class=\"registerdate\">投稿日時</th>\n"
       "\t\t<th class=\"lastregisterdate\">最終更新日時</th>\n"
@@ -306,6 +320,34 @@ void output_ticket_table(SearchResult* result, List* element_types)
     output_ticket_table_body(result, element_types);
     o("</table>\n");
 }
+void output_states(List* states)
+{
+    Iterator* it;
+    /* stateの表示 */
+    o("<div id=\"state_index\">\n");
+    o("\t<ul>\n");
+    foreach (it, states) {
+        State* s = it->element;
+        o("\t\t<li>\n");
+        o("\t\t\t<a href=\"%s/search?field%d=", cgiScriptName, ELEM_ID_STATUS);
+        u(s->name);
+        o("\" title=\"状態を条件にして検索を行ないます。\">");
+        h(s->name);
+        o("</a>");
+        o("(%d)\n", s->count);
+        o("\t\t</li>\n");
+    }
+    o("\t\t<li>\n");
+    o("\t\t\t<form action=\"%s/search\" method=\"get\">\n", cgiScriptName);
+    o("\t\t\t\t<input type=\"text\" class=\"number\" name=\"id\" title=\"入力したIDのチケットを表示します。\" maxlength=\"1000\" />\n"
+      "\t\t\t\t<input type=\"submit\" class=\"button\" value=\"ID指定で表示\" />\n"
+      "\t\t\t</form>\n"
+      "\t\t</li>\n"
+      "\t</ul>\n"
+      "\t<br clear=\"all\" />\n"
+      "</div>\n");
+/*       "<br clear=\"all\" />\n"); */
+}
 /**
  * 一覧を表示するaction。
  */
@@ -345,30 +387,8 @@ void list_action()
     xfree(project_a);
     list_alloc(states_a, State);
     states_a = db_get_states(states_a);
-    /* stateの表示 */
-    o("<div id=\"state_index\">\n");
-    o("\t<ul>\n");
-    foreach (it, states_a) {
-        State* s = it->element;
-        o("\t\t<li>\n");
-        o("\t\t\t<a href=\"%s/search?field%d=", cgiScriptName, ELEM_ID_STATUS);
-        u(s->name);
-        o("\" title=\"状態を条件にして検索を行ないます。\">");
-        h(s->name);
-        o("</a>");
-        o("(%d)", s->count);
-        o("\t\t</li>\n");
-    }
+    output_states(states_a);
     list_free(states_a);
-    o("\t\t<li>\n");
-    o("\t\t\t<form action=\"%s/search\" method=\"get\">\n", cgiScriptName);
-    o("\t\t\t\t<input type=\"text\" class=\"number\" name=\"id\" title=\"入力したIDのチケットを表示します。\" maxlength=\"1000\" />\n"
-      "\t\t\t\t<input type=\"submit\" class=\"button\" value=\"ID指定で表示\" />\n"
-      "\t\t\t</form>\n"
-      "\t\t</li>\n"
-      "\t</ul>\n"
-      "</div>\n"
-      "<br clear=\"all\" />\n");
     fflush(cgiOut);
     o("<div id=\"ticket_list\">\n"
       "<h3>状態別チケット一覧</h3>\n"
@@ -575,61 +595,13 @@ void search_actoin()
     xfree(sort_a);
     list_alloc(states_a, State);
     states_a = db_get_states(states_a);
-    /* stateの表示 */
-    o("<div id=\"state_index\">\n");
-    o("\t<ul>\n");
-    foreach (it, states_a) {
-        State* s = it->element;
-        o("\t\t<li>\n");
-        o("\t\t\t<a href=\"%s/search?field%d=", cgiScriptName, ELEM_ID_STATUS);
-        u(s->name);
-        o("\" title=\"状態を条件にして検索を行ないます。\">");
-        h(s->name);
-        o("</a>");
-        o("(%d)", s->count);
-        o("\t\t</li>\n");
-    }
+    output_states(states_a);
     list_free(states_a);
-    o("\t\t<li>\n");
-    o("\t\t\t<form action=\"%s/search\" method=\"get\">\n", cgiScriptName);
-    o("\t\t\t\t<input type=\"text\" class=\"number\" name=\"id\" title=\"入力したIDのチケットを表示します。\" maxlength=\"1000\" />\n"
-      "\t\t\t\t<input type=\"submit\" class=\"button\" value=\"ID指定で表示\" />\n"
-      "\t\t\t</form>\n"
-      "\t\t</li>\n"
-      "\t</ul>\n"
-      "<br clear=\"all\" />\n"
-      "</div>\n"
-      "<div id=\"condition_form\">\n"
+    o("<div id=\"condition_form\">\n"
       "<h3>検索条件</h3>\n"
       "<div class=\"description\">検索条件を入力して検索ボタンを押してください。</div>\n");
     o("<form action=\"%s/search\" method=\"get\">\n", cgiScriptName);
     o(      "<table summary=\"condition table\">\n");
-    col_index = 1;
-    foreach (it, element_types_a) {
-        ElementType* et = it->element;
-        char name[DEFAULT_LENGTH];
-        char value[DEFAULT_LENGTH];
-        sprintf(name, "field%d", et->id);
-        cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
-
-        if (col_index == 1)
-            o("<tr>\n");
-        o("\t<th>"); h(et->name); o("</th>\n");
-        o("\t<td>\n"); 
-        output_form_element_4_condition(et);
-        o("\t</td>\n");
-        if (!iterator_next(it)) {
-            /* 空いたセルの調整 */
-            int i = col_index;
-            for (; i < 3; i++) {
-                o("\t<th>&nbsp;</th>\n");
-                o("\t<td>&nbsp;</td>\n");
-            }
-        }
-        if (col_index == 3 || !iterator_next(it))
-            o("</tr>\n");
-        col_index = col_index++ == 3 ? 1 : col_index;
-    }
     cgiFormStringNoNewlines("registerdate.from", registerdate_from, DEFAULT_LENGTH);
     cgiFormStringNoNewlines("registerdate.to", registerdate_to, DEFAULT_LENGTH);
     cgiFormStringNoNewlines("lastregisterdate.from", updatedate_from, DEFAULT_LENGTH);
@@ -662,8 +634,34 @@ void search_actoin()
       "\t\t</span>\n"
       "\t\t<div class=\"description\">yyyy-mm-dd形式で入力してください。</div>\n"
       "\t</td>\n"
-      "</tr>\n"
-      "</table>\n"
+      "</tr>\n");
+    col_index = 1;
+    foreach (it, element_types_a) {
+        ElementType* et = it->element;
+        char name[DEFAULT_LENGTH];
+        char value[DEFAULT_LENGTH];
+        sprintf(name, "field%d", et->id);
+        cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
+
+        if (col_index == 1)
+            o("<tr>\n");
+        o("\t<th>"); h(et->name); o("</th>\n");
+        o("\t<td>\n"); 
+        output_form_element_4_condition(et);
+        o("\t</td>\n");
+        if (!iterator_next(it)) {
+            /* 空いたセルの調整 */
+            int i = col_index;
+            for (; i < 3; i++) {
+                o("\t<th>&nbsp;</th>\n");
+                o("\t<td>&nbsp;</td>\n");
+            }
+        }
+        if (col_index == 3 || !iterator_next(it))
+            o("</tr>\n");
+        col_index = col_index++ == 3 ? 1 : col_index;
+    }
+    o("</table>\n"
       "<input class=\"button\" type=\"submit\" value=\"検索\" />"
       "</form>\n"
       "</div>\n");
@@ -1020,6 +1018,7 @@ void output_field_information_js(List* element_types) {
 void register_action()
 {
     Project* project_a = xalloc(sizeof(Project));
+    List* states_a;
     char sender[DEFAULT_LENGTH];
     cgiCookieString(COOKIE_SENDER, sender, DEFAULT_LENGTH);
 
@@ -1027,8 +1026,12 @@ void register_action()
     project_a = db_get_project(project_a);
     output_header(project_a, "チケット登録", "register.js", NAVI_REGISTER);
     output_calendar_js();
-    o(      "<h2>"); h(project_a->name);o(" - チケット登録</h2>\n"
-            "<div id=\"input_form\">\n"
+    o(      "<h2>"); h(project_a->name);o(" - チケット登録</h2>\n");
+    list_alloc(states_a, State);
+    states_a = db_get_states(states_a);
+    output_states(states_a);
+    list_free(states_a);
+    o(      "<div id=\"input_form\">\n"
             "<h3>チケット登録</h3>\n"
             "<div class=\"description\">新規チケットを登録する場合は、以下のフォームを記入し登録ボタンを押してください。</div>\n"
             "<noscript><div class=\"description\">※必須項目の入力チェックは、javascriptで行なっています。</div></noscript>\n");
@@ -1068,7 +1071,7 @@ void register_action()
             "<input id=\"save2cookie\" type=\"checkbox\" name=\"save2cookie\" class=\"checkbox\" value=\"1\" %s />\n"
             "<label for=\"save2cookie\">投稿者を保存する。(cookie使用)</label>\n"
             "</form>\n"
-            "</div>\n", strlen(sender) ? "checked" : "");
+            "</div>\n", strlen(sender) ? "checked=\"checked\"" : "");
     o(      "<div class=\"description\">\n"
             "\t<ul>\n"
             "\t\t<li>複数行テキスト項目の内容では、#1 のように書くと、ID が 1 のチケットのチケット詳細ページへのリンクとなります。</li>\n"
@@ -1083,6 +1086,7 @@ void register_action()
  */
 void ticket_action()
 {
+    String* title_a = string_new(0);
     char path_info[DEFAULT_LENGTH];
     char* ticket_id;
     List* elements_a = NULL;
@@ -1109,16 +1113,18 @@ void ticket_action()
         redirect("/list", "存在しないIDが指定されました。");
         return;
     }
+    string_appendf(title_a, "#%d %s", iid, get_element_value_by_id(elements_a, ELEM_ID_TITLE));
     project_a = db_get_project(project_a);
-    output_header(project_a, "チケット詳細", "reply.js", NAVI_OTHER);
+    output_header(project_a, string_rawstr(title_a), "reply.js", NAVI_OTHER);
     output_calendar_js();
 
     message_ids_a = db_get_message_ids_a(iid);
     list_alloc(element_types_a, ElementType);
     element_types_a = db_get_element_types_all(element_types_a);
-    o("<h2 id=\"subject\">"); h(project_a->name); o(" - #%d ", iid);
+    o("<h2 id=\"subject\">"); h(project_a->name); o(" - ");
     xfree(project_a);
-    h(get_element_value_by_id(elements_a, ELEM_ID_TITLE));
+    h(string_rawstr(title_a));
+    string_free(title_a);
     o(" &nbsp;</h2>\n");
     o(      "<div id=\"ticket_newest\">\n"
             "<h3>チケット最新情報</h3>\n"
@@ -1324,7 +1330,7 @@ void ticket_action()
             "<input id=\"save2cookie\" type=\"checkbox\" name=\"save2cookie\" class=\"checkbox\" value=\"1\" %s />\n"
             "<label for=\"save2cookie\">投稿者を保存する。(cookie使用)</label>\n"
             "</form>\n"
-            "</div>\n", strlen(sender) ? "checked" : "");
+            "</div>\n", strlen(sender) ? "checked=\"checked\"" : "");
     o(      "<div class=\"description\">\n"
             "\t<ul>\n"
             "\t\t<li>複数行テキスト項目の内容では、#1 のように書くと、ID が 1 のチケットのチケット詳細ページへのリンクとなります。</li>\n"
@@ -1396,39 +1402,33 @@ void register_submit_action()
                 case ELEM_TYPE_TEXT:
                 case ELEM_TYPE_DATE:
                     cgiFormStringNoNewlines(name, value_a, VALUE_LENGTH);
-                    e->str_val = xalloc(sizeof(char) * strlen(value_a) + 1);
-                    strcpy(e->str_val, value_a);
+                    set_element_value(e, value_a);
                     break;
                 case ELEM_TYPE_TEXTAREA:
                     cgiFormString(name, value_a, VALUE_LENGTH);
-                    e->str_val = xalloc(sizeof(char) * strlen(value_a) + 1);
-                    strcpy(e->str_val, value_a);
+                    set_element_value(e, value_a);
                     break;
                 case ELEM_TYPE_CHECKBOX:
                     cgiFormString(name, value_a, VALUE_LENGTH);
-                    e->str_val = xalloc(sizeof(char) * strlen(value_a) + 1);
-                    strcpy(e->str_val, value_a);
+                    set_element_value(e, value_a);
                     break;
                 case ELEM_TYPE_LIST_SINGLE:
                     /* 新規選択肢 */
                     cgiFormString(name_new_item, value_a, VALUE_LENGTH);
                     if (strlen(value_a)) {
-                        e->str_val = xalloc(sizeof(char) * strlen(value_a) + 1);
-                        strcpy(e->str_val, value_a);
+                        set_element_value(e, value_a);
                         /* 新しく選択肢を追加 */
                         register_list_item(et->id, value_a);
                     } else {
                         cgiFormString(name, value_a, VALUE_LENGTH);
-                        e->str_val = xalloc(sizeof(char) * strlen(value_a) + 1);
-                        strcpy(e->str_val, value_a);
+                        set_element_value(e, value_a);
                     }
                     break;
                 case ELEM_TYPE_LIST_MULTI:
                     /* 新規選択肢 */
                     cgiFormString(name_new_item, value_a, VALUE_LENGTH);
                     if (strlen(value_a)) {
-                        e->str_val = xalloc(sizeof(char) * strlen(value_a) + 1);
-                        strcpy(e->str_val, value_a);
+                        set_element_value(e, value_a);
                         /* 新しく選択肢を追加 */
                         register_list_item(et->id, value_a);
                     }
@@ -1446,8 +1446,7 @@ void register_submit_action()
                             i++;
                         }
                     }
-                    e->str_val = xalloc(sizeof(char) * strlen(value_a) + 1);
-                    strcpy(e->str_val, value_a);
+                    set_element_value(e, value_a);
                     cgiStringArrayFree(multi);
                     break;
                 case ELEM_TYPE_UPLOADFILE:
@@ -1504,6 +1503,293 @@ file_size_error:
     output_footer();
     xfree(project_a);
 }
+/**
+ * 一括登録画面を表示するaction。
+ */
+void register_at_once_action()
+{
+    Project* project_a = xalloc(sizeof(Project));
+    List* states_a;
+    char sender[DEFAULT_LENGTH];
+    cgiCookieString(COOKIE_SENDER, sender, DEFAULT_LENGTH);
+
+    db_init();
+    project_a = db_get_project(project_a);
+    output_header(project_a, "チケット一括登録", "register_at_once.js", NAVI_REGISTER_AT_ONCE);
+    output_calendar_js();
+    o(      "<h2>"); h(project_a->name);o(" - チケット一括登録</h2>\n");
+    list_alloc(states_a, State);
+    states_a = db_get_states(states_a);
+    output_states(states_a);
+    list_free(states_a);
+    o(      "<div id=\"input_form\">\n"
+            "<h3>チケット一括登録</h3>\n"
+            "<div class=\"description\">\n"
+            "新規チケットを一括登録する場合は、以下のフォームを記入し登録ボタンを押してください。登録の手順は下のとおりです。\n"
+            "\t<ul>\n"
+            "\t\t<li>チケット一括登録ページ(このページ)で、CSVを登録して、解析ボタンを押す。</li>\n"
+            "\t\t<li>チケット一括登録確認ページで、各フィールドに対応する項目を選択し、登録ボタンを押す。</li>\n"
+            "\t\t<li>登録完了。</li>\n"
+            "\t</ul>\n"
+            "</div>\n"
+            "<noscript><div class=\"description\">※必須項目の入力チェックは、javascriptで行なっています。</div></noscript>\n");
+    o(      "<form id=\"register_form\" name=\"register_form\" action=\"%s/register_at_once_confirm\" method=\"post\">\n", cgiScriptName);
+    xfree(project_a);
+    {
+        /* 一括用、CSV形式フィールド */
+        o("<table summary=\"input infomation\">\n"
+          "\t<tr>\n"
+          "\t\t<th class=\"required\">CSV"
+          "<span class=\"required\">※</span>"
+          "</th><td>\n"
+          "\t\t\t<div id=\"csvdata.required\" class=\"error\"></div>\n"
+          "\t\t\t<textarea name=\"csvdata\" id=\"csvdata\" row=\"5\" col=\"5\" wrap=\"off\"></textarea>\n"
+          "\t\t\t<div class=\"description\">登録したいデータをCSV形式で貼り付けてください。次の画面で取り込むフィールドを設定可能です。&nbsp;</div>\n"
+          "\t\t</td>\n"
+          "\t</tr>\n"
+          "</table>\n");
+    }
+    o(      "<input class=\"button\" type=\"submit\" name=\"register\" value=\"解析\" />\n"
+            "</form>\n"
+            "</div>\n");
+    db_finish();
+    output_footer();
+}
+/**
+ * 一括登録確認画面を表示するaction。
+ */
+void register_at_once_confirm_action()
+{
+    int i;
+    int row = 0;
+    Project* project_a = xalloc(sizeof(Project));
+    List* states_a;
+    Csv* csv_a;
+    char sender[DEFAULT_LENGTH];
+    char* content_a = xalloc(sizeof(char) * VALUE_LENGTH);
+    cgiCookieString(COOKIE_SENDER, sender, DEFAULT_LENGTH);
+
+    db_init();
+    d("start\n");
+    cgiFormString("csvdata", content_a, VALUE_LENGTH);
+    d("content_a: %s\n", content_a);
+    csv_a = csv_new(content_a);
+
+
+
+    project_a = db_get_project(project_a);
+    output_header(project_a, "チケット一括登録確認", "register_at_once_submit.js", NAVI_REGISTER_AT_ONCE);
+    output_calendar_js();
+    o(      "<h2>"); h(project_a->name);o(" - チケット一括登録確認</h2>\n");
+    list_alloc(states_a, State);
+    states_a = db_get_states(states_a);
+    output_states(states_a);
+    list_free(states_a);
+    o(      "<div id=\"input_form\">\n"
+            "<h3>チケット一括登録確認</h3>\n"
+            "<div class=\"description\">各桁の登録対象コンボボックスの選択値を、その桁のデータの登録対象となる項目を選択して、登録ボタンを押してください。</div>\n"
+            "<noscript><div class=\"description\">※必須項目の入力チェックは、javascriptで行なっています。</div></noscript>\n");
+    o(      "<form id=\"register_form\" name=\"register_form\" action=\"%s/register_at_once_submit\" method=\"post\">\n", cgiScriptName);
+    o(      "<table summary=\"input infomation\">\n");
+    xfree(project_a);
+    {
+        int line_count = 0;
+        List* element_types_a;
+        Iterator* it;
+        Iterator* it_line;
+        list_alloc(element_types_a, ElementType);
+        element_types_a = db_get_element_types_all(element_types_a);
+        foreach (it, element_types_a) {
+            ElementType* et = it->element;
+            /* 返信専用属性は表示しない。 */
+            if (et->reply_property == 1) continue;
+            /* 投稿者だけを表示する。 */
+            if (et->id != ELEM_ID_SENDER) continue;
+            o("\t<tr>\n");
+            o("\t\t<th %s>", et->required ? "class=\"required\"" : "");
+            h(et->name);
+            if (et->required) {
+                o("<span class=\"required\">※</span>");
+            }
+            o("</th><td>\n");
+            if (et->required)
+                o("\t\t\t<div id=\"field%d.required\" class=\"error\"></div>\n", et->id);
+            output_form_element(NULL, et);
+            o("\t\t\t<div class=\"description\">");h(et->description);o("&nbsp;</div>\n");
+            o("\t\t</td>\n");
+            o("\t</tr>\n");
+        }
+        o("</table>\n");
+        o("<input type=\"hidden\" name=\"fields_count\" value=\"%d\" />\n", csv_a->field_count);
+        o(      "<table id=\"register_at_once_confirm\">\n"
+                "\t<tr>\n");
+        o(  "\t\t<th>登録<br />対象</th>\n");
+        for (i = 0; i < csv_a->field_count; i++) {
+            o(  "\t\t<th>\n");
+            o(  "\t\t\t<select name=\"col_field%d\">\n", i);
+            o(  "\t\t\t\t<option value=\"\">無視</option>\n");
+            foreach (it, element_types_a) {
+                ElementType* et = it->element;
+                if (et->id == ELEM_ID_SENDER) continue;
+                o("\t\t\t\t<option value=\"%d\">\n", et->id);
+                o("\t\t\t\t\t"); h(et->name); o("\n");
+                o("\t\t\t\t</option>\n");
+            }
+            o(  "\t\t\t</select>\n");
+            o(  "\t\t</th>\n");
+        }
+        o(      "\t</tr>\n");
+        /* データ */
+        foreach (it_line, csv_a->lines) {
+            CsvLine* line = it_line->element;
+            Iterator* it_fields;
+            int fields_count = 0, i;
+            d("csv field size %d\n", line->fields->size);
+            o("\t<tr>\n");
+            o("\t\t<th>%d", (row++) + 1);
+            o("</th>\n");
+            foreach (it_fields, line->fields) {
+                CsvField* field = it_fields->element;
+                d("csv col %s\n", string_rawstr(field->data));
+                o("\t\t<td>\n");
+                o("\t\t\t<textarea name=\"csvfield%d.%d\" row=\"5\" col=\"5\">", line_count, fields_count); h(string_rawstr(field->data)); o("</textarea>\n");
+                o("\t\t</td>\n");
+                fields_count++;
+            }
+            for (i = csv_a->field_count; i > fields_count; i--) {
+                o("\t\t<td>&nbsp;</td>\n");
+            }
+            o("\t</tr>\n");
+            line_count++;
+        }
+        o("</table>\n");
+        list_free(element_types_a);
+    }
+    o(      "<input class=\"button\" type=\"submit\" name=\"register\" value=\"登録\" />\n"
+            "<input id=\"save2cookie\" type=\"checkbox\" name=\"save2cookie\" class=\"checkbox\" value=\"1\" %s />\n"
+            "<label for=\"save2cookie\">投稿者を保存する。(cookie使用)</label>\n"
+            "</form>\n", strlen(sender) ? "checked=\"checked\"" : "");
+    o(      "<div class=\"description\">\n"
+            "\t<ul>\n"
+            "\t\t<li>登録対象コンボボックスの選択値を、「無視」のまま登録すると、登録対象から省かれます。</li>\n"
+            "\t\t<li>登録する各値は、入力チェックが行なわれませんので、登録前に確認してください。</li>\n"
+            "\t</ul>\n"
+            "</div>\n");
+    o(      "</div>\n");
+    db_finish();
+    csv_free(csv_a);
+    output_footer();
+}
+void register_at_once_submit_action()
+{
+    int row_count = 0, col_count = 0;
+    Iterator* it;
+    char save2cookie[2];
+    char senderfield[DEFAULT_LENGTH];
+    char sender[DEFAULT_LENGTH];
+    char fields_count_str[5];
+    List* field_ids_a;
+    int i = 0;
+    int fields_count;
+    int registered_tickets_count = 0;
+
+    cgiFormStringNoNewlines("save2cookie", save2cookie, 2);
+    sprintf(senderfield, "field%d", ELEM_ID_SENDER);
+    cgiFormStringNoNewlines(senderfield, sender, DEFAULT_LENGTH);
+    if (strcmp(save2cookie, "1") == 0) {
+        d("set cookie: %s, %s, %s\n", sender, cgiScriptName, cgiServerName);
+        cgiHeaderCookieSetString(COOKIE_SENDER, sender, 86400 * 30, "/", cgiServerName);
+    } else {
+        cgiHeaderCookieSetString(COOKIE_SENDER, "", 0, "/", cgiServerName);
+    }
+
+    cgiFormStringNoNewlines("fields_count", fields_count_str, 5);
+    fields_count = atoi(fields_count_str);
+    /* 項目一覧を取得する。 */
+    list_alloc(field_ids_a, int);
+    for (i = 0; i < fields_count; i++) {
+        char name[DEFAULT_LENGTH];
+        char value[DEFAULT_LENGTH];
+        int* field_id;
+        sprintf(name, "col_field%d", i);
+        cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
+        field_id = list_new_element(field_ids_a);
+        if (strlen(value) != 0) {
+            *field_id = atoi(value);
+        } else {
+            *field_id = -1;
+        }
+        list_add(field_ids_a, field_id);
+    }
+    db_init();
+    db_begin();
+    while (1) {
+        Message* ticket_a = xalloc(sizeof(Message));
+        List* elements_a = NULL;
+        ticket_a->id = -1;
+        list_alloc(elements_a, Element);
+        /* register */
+        col_count = -1;
+        foreach (it, field_ids_a) {
+            int* field_id = it->element;
+            char* value_a;
+            Element* e;
+            value_a = xalloc(sizeof(char) * VALUE_LENGTH); /* 1M */
+            col_count++;
+            if (*field_id == -1) {
+                xfree(value_a);
+                continue;
+            }
+            e = list_new_element(elements_a);
+            char name[DEFAULT_LENGTH] = "";
+            sprintf(name, "csvfield%d.%d", row_count, col_count);
+            strcpy(value_a, "");
+            e->element_type_id = *field_id;
+            e->is_file = 0;
+            cgiFormString(name, value_a, VALUE_LENGTH);
+            if (strlen(value_a) == 0) {
+                /* 値が無ければelementを追加しない */
+                xfree(value_a);
+                continue;
+            }
+            set_element_value(e, value_a);
+            list_add(elements_a, e);
+            if (elements_a->size == 1) {
+                /* 1回だけ投稿者を追加する。 */
+                Element* sender_element = list_new_element(elements_a);
+                sender_element->element_type_id = ELEM_ID_SENDER;
+                set_element_value(sender_element, sender);
+                list_add(elements_a, sender_element);
+            }
+            xfree(value_a);
+        }
+        if (elements_a->size == 0) {
+            /* elementが無い状態だったら、登録処理終了。 */
+            free_element_list(elements_a);
+            xfree(ticket_a);
+            break;
+        }
+        ticket_a->elements = elements_a;
+        ticket_a->id = db_register_ticket(ticket_a);
+        registered_tickets_count++;
+        free_element_list(elements_a);
+        xfree(ticket_a);
+        row_count++;
+    }
+    db_commit();
+    db_finish();
+
+    if (registered_tickets_count) {
+        char message[DEFAULT_LENGTH];
+        sprintf(message, "%d件登録しました。", registered_tickets_count);
+        redirect("/list", message);
+    } else {
+        redirect("/list", "登録されたチケットはありませんでした。戻るボタンで戻って確認してください。");
+    }
+    list_free(field_ids_a);
+    return;
+
+}
 /**
  * デフォルトのaction。
  */
@@ -1664,13 +1950,18 @@ void statistics_action()
     Project* project_a = xalloc(sizeof(Project));
     List* element_types_a;
     Iterator* it;
+    List* states_a;
 
     db_init();
     project_a = db_get_project(project_a);
     output_header(project_a, "統計情報", "graph.js", NAVI_STATISTICS);
     output_graph_js();
-    o(      "<h2>");h(project_a->name);o("</h2>\n"
-            "<div id=\"top\">\n"
+    o(      "<h2>");h(project_a->name);o("</h2>\n");
+    list_alloc(states_a, State);
+    states_a = db_get_states(states_a);
+    output_states(states_a);
+    list_free(states_a);
+    o(      "<div id=\"top\">\n"
             "<h3>統計情報</h3>\n"
             "\t<div class=\"description\">統計情報を表示します。</div>\n");
     xfree(project_a);
